import {Component, EventEmitter, Input, OnInit, Output} from '@angular/core';
import {ModalConfig} from '../modal/modal.config';
import {ModalService} from '../modal/service/modal.service';
import {DEFAULT_INTERRUPTSOURCES, Idle} from '@ng-idle/core';
import {Keepalive} from '@ng-idle/keepalive';
import {CookieService} from '../core/services/cookie.service';
import {ACCESS_TOKEN_KEY, REFRESH_TOKEN_KEY} from '../core/constants';
import {TimeoutService} from './timeout.service';
import { Router } from '@angular/router';


@Component({
  selector: 'collab-comp-timeout',
  templateUrl: './timeout.component.html',
  styleUrls: ['./timeout.component.css']
})

export class TimeoutComponent implements OnInit {

  /**
   * duration of user idle on which a modal dialog is shown
   */
  @Input() idleDuration = 780;
  /**
   * warning for logout is shown for this duration
   */
  @Input() idleWarningDuration = 120;
  /**
   * event emitted on logout of user idle
   */
  @Output() onTimeoutExpired = new EventEmitter();
  @Output() onLogout = new EventEmitter();

  public lastPing?: Date | null = null;
  public idleStateMessage = '';
  public modalConfig: ModalConfig = {
    modalType: 'modal',
    titleSection: {
      title: 'Session Expire Warning',
      titleClose: true
    },
    closeButtonLabel: 'Log Out',
    submitButtonLabel: 'Extend Session',
    disableSubmitButton: false,
  };
  public timedOut = false;

  constructor(private modalService: ModalService, private idle: Idle, private keepalive: Keepalive, private cookieService: CookieService,
    private timeoutService: TimeoutService, private router: Router) {
  }

  ngOnInit(): void {
    this.initializeIdle();
  }

  private initializeIdle(){
    this.idle.setIdle(this.idleDuration);
    this.idle.setTimeout(this.idleWarningDuration);
    // sets the default interrupts, in this case, things like clicks, scrolls, touches to the document
    this.idle.setInterrupts(DEFAULT_INTERRUPTSOURCES);

    this.idle.onIdleEnd.subscribe(() => {
      this.reset();
    });

    this.idle.onTimeout.subscribe(() => {
      this.timedOut = true;
      this.modalService.close('time-out-modal');
      this.onTimeoutExpired.emit();
    });

    this.idle.onTimeoutWarning.subscribe((countdown) => {
      this.idleStateMessage = 'You will be logged out in ' + countdown + ' seconds.';
      this.modalService.open('time-out-modal');
    });

    // sets the ping interval to 15 min
    this.keepalive.interval(900);
    this.keepalive.onPing.subscribe(() => this.lastPing = new Date());

    // to prevent login component from timeout session
    this.router.events.subscribe(() => {
      if (this.router.url.search(/login/) === -1 && this.router.url.search(/password-reset-page/) === -1) {
          this.idle.watch();
      } else {
          this.idle.stop();
      }
    });

    this.reset();
  }

  reset() {
    this.idle.watch();
    this.idleStateMessage = '';
    this.timedOut = false;
  }

  logOut() {
    this.onLogout.emit();
    this.modalService.close('time-out-modal');
   /* if (this.cookieService.getCookie(ACCESS_TOKEN_KEY)){
      this.cookieService.setCookie(ACCESS_TOKEN_KEY, '', 0);
    }*/

    // this.cookieService.setCookie(ACCESS_TOKEN_KEY, '', 0);
    // this.cookieService.setCookie(REFRESH_TOKEN_KEY, '', 0);

  }

  public extendSession() {
    this.reset();
    this.timeoutService.refreshToken().subscribe(data => {
      if (data && data.body) {
        if (data['headers'].get(ACCESS_TOKEN_KEY)) {
          this.cookieService.setCookie(ACCESS_TOKEN_KEY, data['headers'].get(ACCESS_TOKEN_KEY), 30);
        }
        if (data['headers'].get(REFRESH_TOKEN_KEY)) {
          this.cookieService.setCookie(REFRESH_TOKEN_KEY, data['headers'].get(REFRESH_TOKEN_KEY), 30);
        }
      }
      this.reset();
    }, error => {
      console.log(error);
    });
    this.modalService.close('time-out-modal');
  }

}

