import {Directive, Input, ElementRef, HostListener, Renderer2, OnInit} from '@angular/core';


const OFFSET_RIGHT_LEFT = 25;
const OFFSET_TOP_BOTTOM = 15;

@Directive({
    selector: '[tooltip]',
})

export class TooltipDirective implements OnInit  {
    @Input('tooltip') public tooltipTitle!: string;
    @Input() public placement!: string;
    @Input() public delay!: number;
    public tooltip!: HTMLElement | null;
    public offset = OFFSET_RIGHT_LEFT;

    constructor(private el: ElementRef, private renderer: Renderer2) { }

    public ngOnInit() {
        this.el.nativeElement.setAttribute('tabindex', '0');
    }

    @HostListener('mouseover') public onMouseEnter() {
        if (!this.tooltip) { this.showToolTip(); }
    }

    @HostListener('focus') public onFocus() {
        if (!this.tooltip) { this.showToolTip(); }
    }

    @HostListener('mouseleave') public onMouseLeave() {
        if (this.tooltip) { this.hideToolTip(); }
    }

    @HostListener('blur') public onBlur() {
        if (this.tooltip) { this.hideToolTip(); }
    }

    @HostListener('document:keydown.escape', ['$event']) public onKeydownHandler(event: KeyboardEvent) {
        if (this.tooltip) { this.hideToolTip(); }
    }

    private showToolTip() {
        this.createToolTip();
        const { pos } = this.setPositionOfToolTip();
        this.renderer.addClass(this.tooltip, `ng-tooltip-${pos}`);
        this.renderer.addClass(this.tooltip, 'ng-tooltip-show');
    }

    private hideToolTip() {
        this.renderer.removeClass(this.tooltip, 'ng-tooltip-show');
        if (this.tooltip) {
            this.renderer.removeChild(document.body, this.tooltip);
            this.tooltip = null;
        }
    }

    private createToolTip() {
        this.tooltip = this.renderer.createElement('span');

        this.renderer.appendChild(
            this.tooltip,
            this.renderer.createText(this.tooltipTitle) // textNode
        );

        this.renderer.appendChild(document.body, this.tooltip);
        this.renderer.addClass(this.tooltip, 'ng-tooltip');


        // uncomment below lines to add animation on show tooltip.
        // this.renderer.setStyle(this.tooltip, '-webkit-transition', `opacity ${this.delay}ms`);
        // this.renderer.setStyle(this.tooltip, '-moz-transition', `opacity ${this.delay}ms`);
        // this.renderer.setStyle(this.tooltip, '-o-transition', `opacity ${this.delay}ms`);
        // this.renderer.setStyle(this.tooltip, 'transition', `opacity ${this.delay}ms`);
    }

    private setPositionOfToolTip() {
        let pos = this.placement;
        const hostPos = this.el.nativeElement.getBoundingClientRect();
        const tooltipPos = this.tooltip!.getBoundingClientRect();
        const scrollPos = window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop || 0;
        let top, left;

        if (this.placement === 'top') {
            this.offset = OFFSET_TOP_BOTTOM;
            top = hostPos.top - tooltipPos!.height - this.offset;
            left = hostPos.left + (hostPos.width - tooltipPos!.width) / 2;
        }

        if (this.placement === 'bottom') {
            this.offset = OFFSET_TOP_BOTTOM;
            top = hostPos.bottom + this.offset;
            left = hostPos.left + (hostPos.width - tooltipPos.width) / 2;
        }

        if (this.placement === 'left') {
            this.offset = OFFSET_RIGHT_LEFT;
            top = hostPos.top + (hostPos.height - tooltipPos.height) / 2;
            left = hostPos.left - tooltipPos.width - this.offset;
        }

        if (this.placement === 'right') {
            this.offset = OFFSET_RIGHT_LEFT;
            top = hostPos.top + (hostPos.height - tooltipPos.height) / 2;
            left = hostPos.right + this.offset;
        }

        if (this.placement === 'auto') {
            const rightAvailable =  (window.innerWidth - (hostPos.right + this.offset)) > this.tooltip!.offsetWidth;
            const leftAvailable = (hostPos.left - tooltipPos.width - this.offset) > this.tooltip!.offsetWidth;
            const bottomAvailable = tooltipPos.height + hostPos.bottom + this.offset < window.innerHeight;
            const topAvailable = (hostPos.top - tooltipPos.height - this.offset) > this.tooltip!.offsetHeight;

            if (rightAvailable) {
                this.offset = OFFSET_RIGHT_LEFT;
                pos = 'right';
                top = hostPos.top + (hostPos.height - tooltipPos.height) / 2;
                left = hostPos.right + this.offset;
            } else if (leftAvailable) {
                this.offset = OFFSET_RIGHT_LEFT;
                top = hostPos.top + (hostPos.height - tooltipPos.height) / 2;
                left = hostPos.left - tooltipPos.width - this.offset;
                pos = 'left';
            } else if (bottomAvailable) {
                this.offset = OFFSET_TOP_BOTTOM;
                top = hostPos.bottom + this.offset;
                left = hostPos.left + (hostPos.width - tooltipPos.width) / 2;
                pos = 'bottom';
            } else if (topAvailable) {
                this.offset = OFFSET_TOP_BOTTOM;
                top = hostPos.top - tooltipPos.height - this.offset;
                left = hostPos.left + (hostPos.width - tooltipPos.width) / 2;
                pos = 'top';
            } else {
                this.offset = OFFSET_RIGHT_LEFT;
                pos = 'right';
                top = hostPos.top + (hostPos.height - tooltipPos.height) / 2;
                left = hostPos.right + this.offset;
            }
        }

        this.renderer.setStyle(this.tooltip, 'top', `${top + scrollPos}px`);
        this.renderer.setStyle(this.tooltip, 'left', `${left}px`);

        return { pos: pos || this.placement};
    }
}
